%  This script replicates the Figure 1 b) of EZC              
%
% Magic numbers
eta = -6:.1:6;                  
%gamma = 1.5;
%theta = .2;
s = lag(eta,0);
c = mean(eta);

% Declare the Stukel parametrization for function "GL"
%Scenario 1: constant gamma, varying theta 

[GL11] = h_lundter(1, 1, s-c);
[GL21] = h_lundter(1, .5, s-c);
[GL31] = h_lundter(1, .25, s-c);
[GL41] = h_lundter(1, .1, s-c);
[GL41b] = h_lundter(1, .05, s-c);
[GL41c] = h_lundter(1, .025, s-c);
[GL41d] = h_lundter(1, .01, s-c);

[GL12] = h_lundter(1, 1, s-c);
[GL22] = h_lundter(1, 4, s-c);
[GL32] = h_lundter(1, 8, s-c);
[GL42] = h_lundter(1, 20, s-c);
[GL52b] = h_lundter(1, 40, s-c);
[GL52c] = h_lundter(1, 100, s-c);

%Scenario 2: varying gamma, constant theta
%Meaning: flex point constant at (0,x)
[GL51] = h_lundter(.25, .5, s-c);
[GL61] = h_lundter(.5, .5, s-c);
[GL71] = h_lundter(1, .5, s-c);
[GL81] = h_lundter(2, .5, s-c);

[GL52] = h_lundter(.25, 2, s-c);
[GL62] = h_lundter(.5, 2, s-c);
[GL72] = h_lundter(1, 2, s-c);
[GL82] = h_lundter(2, 2, s-c);

%Scenario 3: varying gamma, varying theta 
%Meaning: flex point varying towards SE direction 
[GL91] = h_lundter(.25, .1, s-c);
[GL101] = h_lundter(.5, .5, s-c);
[GL111] = h_lundter(1, 1, s-c);
[GL121] = h_lundter(2, 2, s-c);

[GL92] = h_lundter(.25, 2, s-c);
[GL102] = h_lundter(.5, 4, s-c);
[GL112] = h_lundter(1, .1, s-c);
[GL122] = h_lundter(2, .5, s-c);

% Compute the "Mu" parametrization


%Results
figure(1)
subplot(2,1,1)
p1 = plot(eta, GL11, eta, GL21, eta, GL31, eta, GL41, eta, GL41b, eta,...
    GL41c, eta, GL41d);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT Logistic Model: Scenario 1 (gamma constant, theta <= 1)') 
legend('\theta= 1', '\theta= .5', '\theta= .25', '\theta= .1','\theta= .05',...
    '\theta= .025','\theta= .01', 1);
set(p1,'LineWidth',3)
axis tight;
subplot(2,1,2)
p2 = plot(eta, GL12, eta, GL22, eta, GL32, eta, GL42, eta, GL52b,eta, GL52c);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT Asymmetric Logistic: Scenario 1 (gamma constant, theta >= 1)')  
legend('\theta= 1', '\theta= 2', '\theta= 4', '\theta= 20','\theta= 40','\theta= 100', 1);
set(p2,'LineWidth',2);
axis tight;

figure(2)
subplot(2,1,1)
p3 = plot(eta, GL51, eta, GL61, eta, GL71, eta, GL81);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT Logistic: Scenario 2 (gamma varying, theta= 0.5)')
legend('gamma= .25', 'gamma= .5', 'gamma= 1', 'gamma= 2', 2)
set(p3,'LineWidth',3)
axis tight;
subplot(2,1,2)
p4 = plot(eta, GL52, eta, GL62, eta, GL72, eta, GL82);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT Asymmetric Logistic: Scenario 2 (gamma varying, theta= 2)')  
legend('gamma= 1', 'gamma= 2', 'gamma= 4', 'gamma= 10', 2)
set(p4,'LineWidth',3)
axis tight;



figure(3)
subplot(2,1,1)
p5 = plot(eta, GL91, eta, GL101, eta, GL111, eta, GL121);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT Logistic: Scenario 3 (gamma varying, theta varying)') 
legend('gamma=.25, theta=.1', 'gamma=.5, theta=.5', 'gamma=1, theta=1', 'gamma=2, theta=2', 0)
set(p5,'LineWidth',3)
axis tight;
subplot(2,1,2)
p6 = plot(eta, GL92, eta, GL102, eta, GL112, eta, GL122);
xlabel('eta')                          
ylabel('h(eta)')                       
title('LT: Scenario 3') 
legend('gamma=.25, theta=2', 'gamma=4, theta=.5', 'gamma=1, theta=.1',...
    'gamma=2, theta=.5', 0)
set(p6,'LineWidth',3)
axis tight;

figure(4)
p7 = plot(eta, GL11, eta, GL41, eta, GL41d, ...
          eta, GL42, eta, GL51, eta, GL61,...
          eta, GL71, eta, GL81, eta, GL52,...
          eta, GL52c, eta, GL62,...
          eta, GL72, eta, GL82);
xlabel('X')                          
ylabel('G(\gamma, \theta; X)')                       
title('SLN-LT Transition Function')
legend('\theta= 1, \gamma=1', '\theta= .1, \gamma=1', '\theta= .01, \gamma=1,',...
    '\theta= 20, \gamma=1', '\theta= .5, \gamma= .25', '\theta= .5, \gamma= .5',...
    '\theta= .5, \gamma= 1', '\theta= .5, \gamma= 2', '\theta= 2 \gamma= .25',...
    '\theta=100, \gamma= 1', '\theta=2, \gamma= .5', ...
    '\theta=2, \gamma= 1', '\theta=2, \gamma= 2', 0)

