% This file perform the simulation for test for NO ADDITIVE ASYMMETRY in a GSTAR model
%
clear
clc
randn('state', 0);
%% STEP0: Define Parameters
% Linear Part
phi0 = 0.0;
phi1 = 0.8;
phi2 = -0.7;
% NON-Linear Part
theta0 = 0.01;  
theta1 = -0.9;
theta2 = 0.795;
% ADDITIVE Nonlinear Part
pi0 = 0.02;     %Set to 0 when computing the Size
pi1 = -1.8;     %Set to 0 when computing the Size
pi2 = 1.6;     %Set to 0 when computing the Size
% Slope parameters of G1 
gamma1 = 150;
gamma2 = 70;
% Slope parameters of G2 (the additive nonlinear function)
gamma3 = 500;  %Set to 0 when computing the Size
gamma4 = 200;  %Set to 0 when computing the Size
% Other parameters
T = 100;
nsim = 1000;
alpha01 = 0.01; alpha05 = 0.05; alpha10 = 0.1;
sigerr = 1;
d=100;  % Discard the first 1,...d simulations

%Preallocate
Y = zeros(T-2,nsim);
LM=zeros(1,nsim); LM1=zeros(1, nsim);
P_LM=zeros(1, nsim); P_LM1=zeros(1, nsim);
Beta = zeros(1, nsim);
Power = zeros(1, nsim);

%% STEP2: INITIALIZATION
%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);

%Generate G
[theta, eta, s, c, eta_t, h, mu, G] = sim_G_glstar1(phi0, phi1, phi2, theta0, theta1, theta2, gamma1, gamma2, y, T);

%Build up the elements of the Score and Hessian (for whom I need only the
%parameters gamma1 gamma2, T, eta and eta_t which in turns are function
%of the Linear AR model previously defined.
[Ggamma1] = Ggamma1(gamma1, gamma2, h, eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h, eta, eta_t, T);
[Gc] = Gc(gamma1, gamma2, h, eta, eta_t, T);

%%  STEP2: SIMULATE  
for S=1:nsim
    
y = zeros(T,1);
u = zeros(T,1);

%Generate a linear AR in order to have w which is required for G and the
%Hessian
[y, eps, phi, w, u_l] = sim_nAR(phi0, phi1, phi2, T , sigerr);
u_l=u_l(3:end,1);
 
%Generate G1 and G2
[h, mu, G1]= G1_glstar1(gamma1, gamma2, y);
[h_2, mu_2, G2] = G2_glstar1(gamma3, gamma4, y);

%DEFINE THE GSTAR under ADDITIONAL ASYMMETRY
[y_gstar, y_addgstar, u_addgstar, p, bar_z, Z] = sim_addGSTAR(y, phi0, phi1, phi2, theta0, theta1, theta2, pi0, pi1, pi2, T, G1, G2, eps);
Y(:,S)=y_addgstar;
%%       STEP3: ESTIMATE the GSTAR model Parameters by concentrated OLS 
%                   (Leybourne-Newbold-Vougas (1999)

%Define the matrix of regressors
O=ones(1,p+1);
ZG1=Z'.*kron(O,G1);
%ZG2=Z'.*kron(O,G2);
ZGgamma1=Z'.*kron(O,Ggamma1);
ZGgamma2=Z'.*kron(O,Ggamma2);
ZGc=Z'.*kron(O,Gc);
%Regress and store the etimates and residuals of the model generated UNDER 
%THE NULL HYPOTHESIS OF NO ADDITIVE ASYMMETRY
params_ols = ols(y_addgstar', [Z' ZG1 ZGgamma1 ZGgamma2 ZGc]);
params_est = [params_ols(1:end).beta];
RES = [params_ols(1:end).resid];
SE = [params_ols(1:end).bstd];
SSR=sum(RES.^2);
sigma2 = (1/T)*SSR;

phi0_hat = params_est(1);
phi1_hat = params_est(2);
phi2_hat = params_est(3);
theta0_hat = params_est(4);
theta1_hat = params_est(5);
theta2_hat = params_est(6);
pi0_hat = params_est(7);
pi1_hat = params_est(8);
pi2_hat = params_est(9);

phi_hat = [phi0_hat; phi1_hat; phi2_hat];
theta_hat = [theta0_hat; theta1_hat; theta2_hat];
pi_hat = [pi0_hat; pi1_hat; pi2_hat];

eps1=eps(p+1:end)';

y_hataddgstar = phi_hat'*Z + theta_hat'*Z.*G1' + pi_hat'*Z.*G2' + eps1 ; %Estimated y_gstar(t) under ADD. ASYMM
u_hataddgstar = y_hataddgstar - phi_hat'*Z - theta_hat'*Z.*G1' - pi_hat'*Z.*G2'; %Estimated u_gstar(t) under ADD. ASYMM

%%       STEP5: TESTS
%Define U and ZS=[s,s^2,s^3]  
U = u_hataddgstar';

%Define the matrix of covariates for the alternative of additive asymmetry
V = [s, s.^2, s.^3];         %Covariates for the ALTERNATIVE
V2 = [s, s.^2];              %Covariates for the null of NO GESTAR Additional Asymmetry (H_02)
V1 = s;                      %Covariates for the null of NO GLSTAR Additional Asymmetry (H_01)
% Matrix of covariates UNDER THE NULL of NO ADDITITIVE (GLSTAR/GESTAR) ASYMMETRY. 
z0 = [ Z', ZG1, ZGgamma1, ZGgamma2, ZGc]; 
z01 = [V1 Z', ZG1, ZGgamma1, ZGgamma2, ZGc]; 
z02 = [V2 Z', ZG1, ZGgamma1, ZGgamma2, ZGc];
% Matrix of covariates UNDER THE ALTERNATIVE. 
z1 = [V, Z', ZG1, ZGgamma1, ZGgamma2, ZGc];

%  F-test
%Regress and store the etimates and residuals under H0 - 
%NOTE: WE USE THE HETERO-ROBUST VERSION of
%the test while regressing U=[hat_u_tvgstar(t)] on z.
params0_ols = ols(U, z01);                    
params0_est = [params0_ols(1:end).beta];     
RES0 = [params0_ols(1:end).resid];          
SE0 = [params0_ols(1:end).bstd];             
SSR0 = sum(RES0.^2);                         
sigma20 = (1/T)*SSR0;                        

%Regress and store the etimates and residuals under H1 
params1_ols = ols(U, z1);                    
params1_est = [params1_ols(1:end).beta];
RES1 = [params1_ols(1:end).resid];
SE1 = [params1_ols(1:end).bstd];
SSR1 = sum(RES1.^2);
sigma21 = (1/T)*SSR1;

%F-statistic
n=size(z01,2);
%n = size([Z' G Ggamma1 Ggamma2 Gc],2);
LM = ((SSR0-SSR1)/(p))/(SSR1/(T-n-p)); %REMARK: we assumed to know the additional nonlinear function (a Logistic).
                                       %Set 3*p and T-3*p-n if this is not known in advance.
%Chi2-statistic
LM1 = (SSR0-SSR1)/sigma21;

%Compute the p-value according to the so found statistics
pval_LM = 1-cdf('f', LM, p, T-n-p);
pval_LM1 = 1-cdf('chi2', LM1, p);

%Store the Draws of the Statistic and p-value
LM(1,S) = LM; 
LM1(1,S) = LM1; 
P_LM(1,S) = pval_LM;
P_LM1(1,S) = pval_LM1;
end
%Discard the first d draws
Y(:,1:d)=[];
P_LM(:,1:d)=[]; P_LM1(:,1:d)=[];
%Define the Empirical P-value
EPval_LM= mean(P_LM); EPval_LM1= mean(P_LM1);
%Define the Type II error (Beta) 
NS=1:size(Y,2);
Beta_P01_LM=cumsum(P_LM>alpha01)./NS; Beta_P01_LM1=cumsum(P_LM1>alpha01)./NS;
Beta_P05_LM=cumsum(P_LM>alpha05)./NS; Beta_P05_LM1=cumsum(P_LM1>alpha05)./NS;
Beta_P10_LM=cumsum(P_LM>alpha10)./NS; Beta_P10_LM1=cumsum(P_LM1>alpha10)./NS;
%  Power
Power_P01_LM= 1-Beta_P01_LM; Power_P01_LM1= 1-Beta_P01_LM1;
Power_P05_LM= 1-Beta_P05_LM; Power_P05_LM1= 1-Beta_P05_LM1;
Power_P10_LM= 1-Beta_P10_LM; Power_P10_LM1= 1-Beta_P10_LM1;
%  Empirical Beta
EBeta_P01_LM= mean(Beta_P01_LM); EBeta_P01_LM1= mean(Beta_P01_LM1);
EBeta_P05_LM= mean(Beta_P05_LM); EBeta_P05_LM1= mean(Beta_P05_LM1);
EBeta_P10_LM= mean(Beta_P10_LM); EBeta_P10_LM1= mean(Beta_P10_LM1);
%  Empirical Power
EP_P01_LM=mean(Power_P01_LM); EP_P01_LM1=mean(Power_P01_LM1);
EP_P05_LM=mean(Power_P05_LM); EP_P05_LM1=mean(Power_P05_LM1);
EP_P10_LM=mean(Power_P10_LM); EP_P10_LM1=mean(Power_P10_LM1);


disp('*******************************************************************')
disp('                         Monte Carlo Results                       ')
disp('                        NO ADDITIVE ASYMMETRY                      ')
disp('*******************************************************************')
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.01                      ')
disp('-------------------------------------------------------------------')
disp('                                                             ')
disp('             F_LM-statistic     |       Chi^2-statistic      ')
disp('------------------------------- | ---------------------------')
disp(' Empirical  |  Emp.   |   Emp.  |   Emp.  |   Emp.  |  Emp.  ')
disp('  p-value   |  Beta   |  Power  | p-value |  Beta   | Power  ')
disp('    ------  | ------  | ------  | ------  | ------  | ------ ')
disp( [EPval_LM EBeta_P01_LM EP_P01_LM EPval_LM1 EBeta_P01_LM1 EP_P01_LM1])
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.05                            ')
disp('-------------------------------------------------------------------')
disp('                          ')
disp('             F_LM-statistic     |       Chi^2-statistic      ')
disp('------------------------------- | ---------------------------')
disp(' Empirical  |  Emp.   |   Emp.  |   Emp.  |   Emp.  |  Emp. ')
disp('  p-value   |  Beta   |  Power  | p-value |  Beta   | Power  ')
disp('    ------  | ------  | ------  | ------  | ------  | ------ ')
disp( [EPval_LM EBeta_P05_LM EP_P05_LM EPval_LM1 EBeta_P05_LM1 EP_P05_LM1])
disp('-------------------------------------------------------------------')
disp('                           alpha = 0.10                            ')
disp('-------------------------------------------------------------------')
disp('                          ')
disp('             F_LM-statistic     |       Chi^2-statistic      ')
disp('------------------------------- | ---------------------------')
disp(' Empirical  |  Emp.   |   Emp.  |   Emp.  |   Emp.  |  Emp.  ')
disp('  p-value   |  Beta   |  Power  | p-value |  Beta   | Power  ')
disp('    ------  | ------  | ------  | ------  | ------  | ------ ')
disp( [EPval_LM EBeta_P10_LM EP_P10_LM EPval_LM1 EBeta_P10_LM1 EP_P10_LM1])
disp('-------------------------------------------------------------------')

figure(1)
plot(NS, Beta_P01_LM, NS, Power_P01_LM,'-o', NS, Beta_P01_LM1, NS, Power_P01_LM1,'-o');
legend('Beta LM (\alpha=0.01)', 'Power LM (\alpha=0.01)', 'Beta LM1 (\alpha=0.01)', 'Power LM1 (\alpha=0.01)');