% This file replicates the empirical application of GSTAR model for real
% data in Zanetti Chini (2018)
clear all
clc

%% STEP 0: IMPORT AND TRANSFORM DATA 

% Import data
importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/Data_q.xls')
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/mUSIIP.xls');
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/mUSUN.xls');
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/lynx.xls');
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/ssn_yearly.xls');
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/SmoothSSN.xls')
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/Entry.xls');
%importfile('/Users/emiliozanettichini/Desktop/WP/GSTR/Data/Exit.xls');
%
% Selecting and filtering
%y = 100*(log(IIP) - lag(log(IIP),4)); y=y(5:end);                  % US IIP in monthly/quarterly/yearly growth rate
y = 100*((log(UNRATE) - lag(log(UNRATE),4))); y=y(9:end);           % US UNEMPLOYEMNT RATE in levels 
%y = log10(LYNX);                                                   % Canadian Lynx
%y = sqrt(SSN(1:end-3));                                            % Yearly SSN (use square root transformation)
%y = log(MSmoothSSN(1213:end));                                     % Monthly Smoothed SSN (log tansformation)
%y = 100*(log(Entry) - lag(log(Entry),4)); y=y(5:end);
%y = 100*(log(Exit) - lag(log(Exit),4)); y=y(5:end);

%Some general parameters
T = length(y);
c = mean(y);

%% STEP 1: DESCRIPTIVE STATISTICS AND All-In-One TEST
%Descriptive statstics
min = min(y);                         % Minimum
max = max(y);                         % Maximum
ave = mean(y);                        % Mean
med = median(y);                      % Median
mad = mad(y);                         % MAD
std = std(y);                         % Sandard Deviation
skew= skewness(y);                    % Skewness
k   = kurtosis(y);                    % Kurthosis
[h_jb,p_jb] = jbtest(y);              % Jarque-Brera test
[archstat,pval_archstat] = arch(y,2); % McLeod-Li test for ARCH-Effect
 %Regression analysis for DW test
x=[ones(T,1),lag(y,1),lag(y,2)  ];      % (define a covariate matrix)
results  = ols(y,x);                  
R= results.resid;
[P_dw,DW] = dwtest(R,x);              % Durbin Watson test
[h_ks,p_ks,ksstat,cv] = kstest(y);    % Kolmogorov-Smirnov test
[h_z,p_z] = ztest(y,mean(y),0.05);    % z-test
pp = phillips(y,x,1);                 % Phillips-Perron test for unit root

% All-in-One test
s=lag(y,1);                           % Set s according to the minimum p-val in All-in-One test (see q in the next STEP 2)
[S2, pval_S2] = stest_aio(y, T, s);   % 4 for mIP and 1 for mUN 3 for qIP and 1 for qUN


%% STEP 2: ESTIMATE THE MODEL
O = ones(length(y),1);
X = [O, lag(y,1) lag(y,2) lag(y,3) lag(y,4) ]; %
W = [];         
q = lag(y,1);   
T = length(y);
nX = size(X,2); % # of covariates
nW = 0;         % # of Exogenous variables
M = [];         % Max # of Regimes (if empty the function compute it automatically)
rob = 1;        % HAC-robust test ?
flag = 1;       % the q is part of the covariates ?
sig = 0.01;      % nominal size (0.01 for IP and UN, 0.5 for SPR, 0.05 for LYNX)
m=1; 
%initial values
gamma_0=0;
gamma1_0=0;
gamma2_0=0;
p = mbic(y, O, X, 12);
% Call the function for MR-STAR model
[yhat,ehat,alpha,beta,lambda,gamma,c,se,sigma,G,fX,pval_lin] = mrstar(y,X,W,q,T,nX,nW,M,rob,flag,sig);

% Call the function for GSTAR model
[alpha_gstar,beta_gstar,lambda_gstar,gamma1,gamma2,c_gstar,fX_gstar,yhat_gstar,ehat_gstar,G_gstar,sigma_gstar,se_gstar]=parestGSTARlm(y,X,W,q,T,nX,nW,m,gamma1_0,gamma2_0,c(1));

% Call other function for the Score Matrix and further results
[eta, h_stuk, MU_stuk, G_stuk] = G_stukel(s-c_gstar, gamma1, gamma2);
eta=yhat_gstar';
eta_t=[q-c_gstar];
[Ggamma1] = Ggamma1(gamma1, gamma2, h_stuk,  eta, eta_t, T);
[Ggamma2] = Ggamma2(gamma1, gamma2, h_stuk, eta, eta_t,T);
[Gc] = Gc(gamma1, gamma2, h_stuk, eta, eta_t,T);
[S1, pval_S1] = stest_ts(X, lambda_gstar, Ggamma1, Ggamma2, ehat,T);
[LogLik_mrstar] = LLF(T,sigma,ehat);
[LogLik_gstar] = LLF(T,sigma_gstar,ehat_gstar);
[R2] = R2(y, ehat_gstar);
[pval_lr] = LRtest(LogLik_gstar,LogLik_mrstar, 1);
%% STEP 3: DIAGNOSTIC TESTS

% CASE1: MRSTAR
% No error Autocorrelation
LM_STARsc=zeros(10,1);pval_STARsc=zeros(10,1);
for q=1:10
    
    %[LM_sc(q), pval_sc(q)] = dtest_sc(ehat, q, X, G_stuk, Ggamma1, Ggamma2, Gc);
    [LM_STARsc(q), pval_STARsc(q)] = dtest_STARsc(ehat, q, X, G_stuk, Ggamma1, Gc);
end
% No additive asymmetry
[LM_STARnoadd, pval_STARnoadd] = dtest_noaddasy_STAR(ehat, s, X, T, G_stuk, Ggamma1, Gc);
% Parameter constancy
[LM1, LM2, LM3, pval_LM1, pval_LM2, pval_LM3] = dtest_paramconst(ehat, X, T, G_stuk, Ggamma1, Ggamma2, Gc);

% CASE2: GSTAR
% No error Autocorrelation
for q=1:10
    LM_sc=zeros(10,1);pval_sc=zeros(10,1);
    [LM_sc_gstar(q), pval_sc_gstar(q)] = dtest_sc(ehat_gstar, q, X, G_stuk, Ggamma1, Ggamma2, Gc);
end
% No additive asymmetry
[LM_GSTARnoadd, pval_GSTARnoadd] = dtest_noaddasy_GSTAR(ehat, s, X, T, G_stuk, Ggamma1, Ggamma2, Gc);
% Parameter constancy
[LM1_gstar, LM2_gstar, LM3_gstar, pval_LM1_gstar, pval_LM2_gstar, pval_LM3_gstar] = dtest_paramconst(ehat_gstar, X, T, G_stuk, Ggamma1, Ggamma2, Gc);

%% STEP 4: OUTPUT

% Define the arrays of the results (sometimes not compatible with resulting arrays, look at the Workspace eventually)
Gamma = zeros(length(lambda),1);          seGamma = zeros(length(lambda_gstar),1);
Gamma1 = zeros(length(lambda),1);         seGamma1 = zeros(length(lambda_gstar),1);
Gamma2 = zeros(length(lambda),1);         seGamma2 = zeros(length(lambda_gstar),1);
C=zeros(length(lambda),1);                seC =   zeros(length(lambda_gstar),1);     
C_gstar     = zeros(length(lambda),1);    seC_gstar =   zeros(length(lambda_gstar),1); 
Gamma(1:length(gamma),1) = gamma;         seGamma(1,1) = se(1*length(lambda)+1); %seGamma(2,1) = se(3*length(lambda)+2); %seGamma(3,1) = se(3*length(lambda)+3);
C(1:length(c),1) = c;                     seC(1,1) = se(1*length(lambda)+2);  %seC(2,1) = se(3*length(lambda)+4); %seC(3,1) = se(3*length(lambda)+6);
Gamma1(1,1) = gamma1;                     seGamma1(1,1) = se_gstar(1*length(lambda_gstar)+1);
Gamma2(1,1) = gamma2;                     seGamma2(1,1) = se_gstar(1*length(lambda_gstar)+2);
C_gstar(1,1) = c_gstar;                   seC_gstar(1,1) = se_gstar(1*length(lambda_gstar)+3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  DISPLAY THE TABLE   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%                 (eventually, adapt in case of more than 3 Regimes (2 transition functions) ) %%%%%%%%%%%%%%%%%%%%
disp('************************************************************************************************************************************')
disp('                                                           EMPIRICAL RESULTS                                                        ')
disp('************************************************************************************************************************************')
disp('                                                            DIAGNOSTIC TESTS                                                        ')
disp('------------------------------------------------------------------------------------------------------------------------------------')
disp('   Minimum  | Maximum | Average | Median |  MAD   | St. Dev |   Skew  |  Kurth  |J-B test | ARCH-t  | DW test | K-S test| z-Test  |PP test ')
disp('            |         |         |        |        |         |         |         | (p-val) | (p-val) | (p-val) | (p-val) | (p-val) |(t-stat)')
disp('   -------  | ------- | ------- | ------ | ------ | ------- |  ------ |  ------ | ------- | ------- |  ------ | ------- | ------- | -------')
disp( [min max ave med mad std skew k p_jb pval_archstat P_dw p_ks p_z pp.pstat])
disp('------------------------------------------------------------------------------------------------------------------------------------')
disp('                         LINEARITY/SYMMETRY TESTS  (p-values)                       ')
disp('------------------------------------------------------------------------------------------------------------------------------------')
disp(' Linearity| Linearity|All-in-One| Two-Step  ')
disp(' (TRANS 1)| (TRANS 2)|          | Two-Step  ')
disp('   -------   -------   --------   ---------')
disp( [pval_lin  pval_S2 pval_S1])
disp('------------------------------------------------------------------------------------------------------')
disp('------------------------------------------------------------------------------------------------------')
disp('                                               ESTIMATES                                            ')
disp('    ---------------------------------------------- || ------------------------------------------------')
disp('                           MRSTAR                  ||                      GSTAR                      ')
disp('    ---------------------------------------------- || ------------------------------------------------')
disp('                           Values                  ||                      Values                ')
disp('    ---------------------------------------------- || ------------------------------------------------')
disp('      phi   | theta(1)| theta(2) |gamma_k |  c_k   ||   phi   | theta   | gamma_1 | gamma_2  |    c   ')
disp('    ------- |  ------ |  ------  | ------ | -------|| ------- | ------- | ------- | -------  |  ------')
disp( [alpha  lambda(:,1)   Gamma C  alpha_gstar  lambda_gstar Gamma1 Gamma2 C_gstar])
disp('    ---------------------------------------------- || ------------------------------------------------')
disp('                      Standard Errors              ||                  Standard Errors                ')
disp('    ---------------------------------------------- || ------------------------------------------------')
disp('      phi   | theta(1)| theta(2) |gamma_k |  c_k   ||   phi   | theta   | gamma_1 | gamma_2  |    c   ')
disp('    ------- |  ------ |  ------  | ------ | -------|| ------- | ------- | ------- | -------  |  ------')
disp( [se(1:5) se(6:10)   seGamma seC se_gstar(1:5) se_gstar(6:10) seGamma1 seGamma2 seC_gstar])
disp('-------------------------------------------------------------------------------------------')
disp('                                          DIAGNOSTIC TESTS                                 ')
disp('-------------------------------------------------------------------------------------------')
disp('                                             MRSTAR                                        ')
disp('    ---------------------------------------------------------------------------------------')
disp('          No error autocorrelation       |No Add Asym|   Parameter constancy    ')
disp('    -------------------------------------|           |--------------------------')
disp('      q=1   |   q=2   |    q=4   | q=10  |           |  LM_1  |  LM_2   |  LM_3 ')
disp('    --------|---------|----------|------ | --------- |--------|---------|-------')
disp( [ pval_STARsc(1) pval_STARsc(2) pval_STARsc(4) pval_STARsc(10) pval_STARnoadd pval_LM1 pval_LM2 pval_LM3 ])
disp('    ---------------------------------------------------------------------------------------')
disp('    ---------------------------------------------------------------------------------------')
disp('                                             GSTAR                                        ')
disp('    ---------------------------------------------------------------------------------------')
disp('          No error autocorrelation       |No Add Asym|   Parameter constancy    ')
disp('    -------------------------------------|           |--------------------------')
disp('      q=1   |   q=2   |    q=4   | q=10  |           |  LM_1  |  LM_2   |  LM_3 ')
disp('    --------|---------|----------|------ | --------- |--------|---------|-------')
disp( [pval_sc_gstar(1) pval_sc_gstar(2) pval_sc_gstar(4) pval_sc_gstar(10) pval_GSTARnoadd pval_LM1_gstar pval_LM2_gstar pval_LM3_gstar ])
disp('-------------------------------------------------------------------------------------------')

%%%%%%%%%%%%%%%%%%%%%%%        DISPLAY  GRAPHS       %%%%%%%%%%%%%%%%%%%%%%
%FIGURE 1
figure(1)
% Select a starting date:
startDate = datenum('01-31-1947');
% Select an ending date:
endDate = datenum('03-31-2013');
% Create xdata to correspond to the number of months between the start and end dates:
xData = linspace(startDate,endDate,T-1);
% For this example, plot random data:
hl1 = line(xData, y(2:end),'LineWidth',2.0,'Color','b');
ax1 = gca;
ylabel('100*\Delta^4 log(y_t)')
set(ax1,'XColor','k','YColor','k')
legend('Data',1)
% Convert the x tick labels to month names, keeping the limits of sample by using the 'keeplimits' option:
datetick('x','YYYY','keeplimits')
% Then create new axes:
ax2 = axes('Position',get(ax1,'Position'),...
           'YAxisLocation','right',...
           'Color','none',...
           'XColor','k','YColor','k');
% Select a starting date:
startDate = datenum('01-31-1947');
% Select an ending date:
endDate = datenum('03-31-2013');
% Create xdata to correspond to the number of months between the start and end dates:
xData = linspace(startDate,endDate,T-1);
hl2 = line(xData,fX(2:end,1),'Color','r','LineStyle','-.','LineWidth',.5,'Marker','+','MarkerSize',2.0,'Parent',ax2);
hl3 = line(xData,fX(2:end,2),'Color','g','LineStyle','-.','LineWidth',.5,'Marker','x','MarkerSize',4.0,'Parent',ax2);
%hl4 = line(xData,fX(2:end,3),'Color','c','LineStyle','-.','LineWidth',.5,'Marker','d','MarkerSize',2.0,'Parent',ax2);
hl4 = line(xData,fX_gstar(2:end),'Color','k','LineStyle','-','LineWidth',1,'Marker','o','MarkerSize',6,'Parent',ax2);
datetick('x','YYYY','keeplimits')
xlabel('DATE')
title('DATA AND TRANSITION FUNCTION')
legend( 'G_1^{MRSTAR}','G_2^{MRSTAR}', 'G^{GSTAR}',1)
%
% FIGURE 2
figure(2)
p=plot(sort(s(2:end)),sort(fX(2:end,1)), sort(s(2:end)),sort(fX(2:end,2)), sort(s(2:end)),sort(G_stuk(2:end)));
xlabel('s_t')
ylabel('G(s_t, c, h(.))')
title('TRANSITION FUNCTION vs TRANSITION VARIABLE')
legend('G_1^{MRSTAR}', 'G_2^{MRSTAR}', 'G^{GSTAR}',0)
set(p(1),'LineWidth',1, 'Color','r', 'LineStyle', '-.', 'Marker','+','markerSize',4)
set(p(2),'LineWidth',.5, 'Color','g', 'LineStyle', '-.', 'Marker','sq','markerSize',4)
%set(p(3),'LineWidth',.5, 'Color','k', 'LineStyle', '-', 'Marker','o','markerSize',6)
set(p(3),'LineWidth',1, 'Color','k', 'LineStyle', '-','Marker','o','markerSize',6)

% FIGURE 3


