
function [eta, s, c, eta_t, h, MU, G, y, y_ave, eps_ave, sigma2_hat, sd...
    bar_w, bar_z,u] = sim_h_gamma1(phi, gamma1, theta, gamma2, T, nsim)
%This function simulate a time series following the Generalized Logistic
%function by Stukel (1988)
%
%The process is: y_t = phi_1 y_t-1 + theta_1 G + eps.
%
%Notice: phi_10 = theta_10 = 0 in order to simplify.
%
%
%Preallocate


y = zeros(T+1,nsim);
eps = randn(T+1,nsim);
eta = random('unif', -10 ,10, T+2, nsim);
%Simulate eta 
    for ns =1:nsim  
        
        s = mlag(eta(:,ns));
        s=s(2:end);
        eta_ave = mean(eta(:,ns));
        c = repmat(eta_ave,T+1,ns);
        eta_t = (s - c);
                                
       %Define the Stukel function (fromula 2-3)
        h11 = (gamma1^(-1)).*(exp(gamma1*abs(eta_t)) - 1) ; 
        h12 = eta_t;
        h13 = (-gamma1^(-1))*log(1 - gamma1*abs(eta_t)) ; 

        h21 = (-gamma2^(-1)).*(exp(gamma2*abs(eta_t)) - 1) ;
        h22 = eta_t ;
        h23 = (gamma2^(-1))*log(1 - gamma2*abs(eta_t)) ; 
         
        if gamma1 > 0
            h1 = h11;
        elseif gamma1 == 0
            h1 = h12;
        else h1 = h13;
        end
        
        if gamma2 > 0
            h2 = h21;
        elseif gamma2 == 0
            h2 = h22 ;
        else h2 = h23;
        end
        
        %Define: h_gamma, mu_gamma and 

        h = ((eta_t) <=0).*(h2) + ((eta_t) >=0).*(h1);
        MU = exp(h)./(1 + exp(h));
        G = (1 + exp(-MU.*(eta_t))).^(-1);
        
        %Simulate the GSTAR(1) model

        for i= 2 : T               %loop for the construction of te GSTAR(1), index i
            y(i,ns) = phi.*y(i-1,ns) + theta.*G(i-1) + eps(i, ns);
        end
    end
    
    % Compute the average of the simulated series

    %y(i,ns);
    y=y(2:end,:);
    y_ave = y;               %compute the average of the series over the simulations
    G_ave = G;
    eps_ave = mean(eps,2);           %compute the average of the errors over the simulations
    bar_w = y_ave(1:T,:);
    bar_z = [1, bar_w']';

    %u = y_ave - phi'*bar_z - theta'*G_ave;   %compute the estimated residuals
    %sigma2_hat = (1/T)*sum((u.^2));  %compute the variance of the errors over the simulations
    %sd = sqrt(sigma2_hat);       %compute the SD of the errors over the simulations
    %sim_sample_ave = mean(y_ave);    %compute the simulated sample mean
    %sim_sample_var = var(y_ave);     %compute the simulated sample variance
    %sim_sample_sd = sqrt(sim_sample_var); %compute the simulated SD of the sample
    %sim_res_ave = mean(u);     %compute the simulated errors mean
    %sim_res_var = var(u);      %compute the simulated errors variance
    
    
    y;                      %store the results for each observation
    %X_sim = [sim_sample_ave, sim_sample_var, sim_res_ave, sim_res_var];  %store the sample moments of the sample
 

   % save('Sim_Data', 'X');
    


    
    
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RESULTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


disp('    y_ave^s   eps_ave^s   eps_var^s')
disp('    --------------------------- ' )
disp(  y   )

disp(' Simulated sample average: ') 
disp(X_sim(:,1))
disp('Simulated sample variance: ') 
disp(X_sim(:,2))
disp( 'Simulated Residuals: ')
disp(X_sim(:,3) )
disp('Simulated  Residuals variance: ' )
disp(X_sim(:,4)')


figure(1)
title('Simulated GSTAR1(1)')
subplot(2,2,1)
plot(X(:,1));
xlabel('T')                          
ylabel('y^{sim}')                       
title('Conditional mean')
%legend('phi= 0.5', 'theta= 0.8', 'gamma1= 0.7',  'gamma2= 2.0', 's= y_{t-1}', 'c= mean(y_t)', 'T= 100', 'S= 100', 3)
subplot(2,2,2)
plot(X(:,2));
xlabel('T')                          
ylabel('y^{sim}')                       
title('Residuals')
%legend('phi= 0.5', 'theta= 0.8', 'gamma1= 0.7',  'gamma2= 2.0', 's= y_{t-1}', 'c= mean(y_t)', 'T= 100', 'S= 100', 3)
subplot(2,2,3)
qqplot(X(:,1));
xlabel('T')                          
ylabel('y^{sim}')                       
title('QQ plot')
%legend('phi= 0.5', 'theta= 0.8', 'gamma1= 0.7',  'gamma2= 2.0', 's= y_{t-1}', 'c= mean(y_t)', 'T= 100', 'S= 100', 3)
subplot(2,2,4)
cdfplot(X(:,1))
hold on
x = -10:0.1:10;
f = normcdf(x,0,1);
plot(x,f,'m')
legend('Empirical','Theoretical', 2)



end
